#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/mman.h>

#include <linux/videodev2.h>

#define NR_BUF 1

static void
die(char *message)
{
  perror(message);
  exit(1);
}

int main(int argc, char *argv[])
{
  struct v4l2_capability cap;
  struct v4l2_requestbuffers req;
  struct v4l2_buffer buf;
  void *buf_addrs[NR_BUF];
  void *addr;
  int ret;
  int fd;
  int i;
  int out;

  memset(&cap, 0, sizeof(cap));
  memset(&req, 0, sizeof(req));
  memset(&buf, 0, sizeof(buf));

  fd = open("/dev/video0", O_RDONLY);
  if (fd < 0)
    die("open");

  ret = ioctl(fd, VIDIOC_QUERYCAP, &cap);
  if (ret < 0)
    die("query capability");
  if (!(cap.capabilities & V4L2_CAP_STREAMING))
    die("no streaming capability");

  req.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
  req.count = NR_BUF;
  req.memory = V4L2_MEMORY_MMAP;
  ret = ioctl(fd, VIDIOC_REQBUFS, &req);
  if (ret < 0)
    die("request buf (mmap)");

  printf("req buf: count %d\n", req.count);
  for (i=0; i<req.count; i++) {
    buf.index = i;
    buf.type = req.type;
    buf.memory = req.memory;

    ret = ioctl(fd, VIDIOC_QUERYBUF, &buf);
    if (ret < 0)
      die("query buf");

    buf_addrs[i] = mmap(NULL, buf.length,
			PROT_READ, MAP_SHARED,
			fd, buf.m.offset);

    printf("queuing buf %i\n", buf.index);
    ioctl(fd, VIDIOC_QBUF, &buf);
    if (ret < 0)
      die("queue buf");
  }
  ret = ioctl(fd, VIDIOC_STREAMON, &(buf.type));
  if (ret < 0)
    die("stream on");

  ret = ioctl(fd, VIDIOC_DQBUF, &buf);
  if (ret < 0)
    die("dequeue buf");

  addr = buf_addrs[buf.index];

  out = open("/tmp/a.jpg",
	     O_WRONLY|O_CREAT|O_TRUNC,
	     S_IRUSR|S_IWUSR);
  write(out, addr, buf.bytesused);
  close (out);

  close(fd);

  return 0;
}
