#include <SDL.h>
#include <SDL_image.h>

#define SCREEN_WIDTH  (640)
#define SCREEN_HEIGHT (480)

static void
die(char *message)
{
  perror(message);
  exit(1);
}

static void
handle_keyup(SDL_KeyboardEvent *ev, int *done)
{
  SDL_keysym *keysym;
  SDL_Event quit_ev;

  keysym = &(ev->keysym);
  switch (keysym->sym) {
  default:
    memset(&quit_ev, 0, sizeof(quit_ev));
    quit_ev.type = SDL_QUIT;
    SDL_PushEvent(&quit_ev);
    break;
  case SDLK_UP:
    printf("up\n");
    break;
  case SDLK_DOWN:
    printf("down\n");
    break;
  case SDLK_RIGHT:
    printf("right\n");
    break;
  case SDLK_LEFT:
    printf("left\n");
    break;
  }
}

int main(int argc, char *argv[])
{
  SDL_Surface *s;
  SDL_Surface *image;
  int ret;
  int done = 0;

  ret = SDL_Init(SDL_INIT_VIDEO);
  if (ret < 0)
    die("sdl init");
  atexit(SDL_Quit);

  image = IMG_Load("sapporo.jpg");
  if (!image)
    die("failed to load image\n");

  s = SDL_SetVideoMode(SCREEN_WIDTH,
		       SCREEN_HEIGHT,
		       24, SDL_FULLSCREEN);
  if (!s)
    die("failed to set mode\n");

  SDL_BlitSurface(image, NULL, s, NULL);
  SDL_Flip(s);

  while (!done) {
    int ok;
    SDL_Event ev;

    ok = SDL_WaitEvent(&ev);
    if (!ok)
      break;
    switch (ev.type) {
    case SDL_KEYUP:
      handle_keyup((SDL_KeyboardEvent *)&ev,
		   &done);
      break;
    case SDL_QUIT:
      printf("quiting...\n");
      done = 1;
      break;
    default:
      printf("ev %d\n", ev.type);
      break;
    }
  }

  return 0;
}
